/*
 * @Author       : Eleanor Mao
 * @Date         : 2024-04-11 10:14:35
 * @LastEditTime : 2024-04-11 10:14:35
 *
 * Copyright © RingCentral. All rights reserved.
 */
import axios from 'axios';
import type { AxiosHeaders, AxiosError } from 'axios';
import qs from 'qs';
import { isFunction } from 'lodash';

type Params = {
    [key: string]: string
}

interface Props {
    urlParams?: {
        [key: string]: string
    };
    paramsSerializer?: (params: Params) => string;
    timeoutCallback?: (error: any) => void;
    cancelCallback?: (error: any) => void;
    arrayFormat?: 'repeat' | 'brackets' | 'comma' | 'indices';
    allowDots?: boolean;
    headers?: AxiosHeaders;
}

type IFunc = (url: string, data?: any, options?: Props) => Promise<any>

export interface Ajax {
    post?: IFunc;
    get?: IFunc;
    put?: IFunc;
    delete?: IFunc;
    patch?: IFunc;
}

type METHODS = ['post', 'get', 'put', 'delete', 'patch']

const methods: METHODS = ['post', 'get', 'put', 'delete', 'patch'];
const ins: Ajax = {};


axios.defaults.headers.post['Content-Type'] = 'application/json';
axios.defaults.headers.get.Accept = 'application/json';
axios.defaults.timeout = 9500;

const format = function (
    url: string,
    obj: {
        [key: string]: string
    }
) {
    let ret = url;
    for (const key of Object.keys(obj)) {
        ret = ret.replace('{' + key + '}', obj[key]);
    }
    return ret;
};


methods.forEach(method => {
    ins[method] = (url, data, options = {}) => {
        data = data || {};


        if (options.urlParams) {
            url = format(url, options.urlParams);
        }

        if (method === 'get') {
            data = { params: data };
        } else {
            if (
                !(data instanceof FormData) &&
                options.headers &&
                options.headers['Content-Type'] === 'application/x-www-form-urlencoded'
            ) {
                data = qs.stringify(data, {
                    arrayFormat: options.arrayFormat ? options.arrayFormat : 'repeat',
                    allowDots: options.allowDots
                });
            }
            data = { data };
        }

        const promise = new Promise((resolve, reject) => {
            return axios({
                url,
                method,
                ...data,
                withCredentials: true,
                timeout: 9500,
                ...options
            })
                .then(response => {
                    return response.data;
                })
                .then(response => {
                    if (response instanceof Blob) {
                        resolve(response);
                    }
                    if (response.code === 0 || response.code === 200) {
                        resolve(response.data);
                    } else {
                        reject(response);
                    }
                })
                .catch((error: AxiosError) => {
                    if (axios.isCancel(error)) {
                        return isFunction(options.cancelCallback) ? options.cancelCallback(error) : undefined;
                    }

                    // https://github.com/axios/axios/issues/660
                    // @ts-ignore
                    if (error.code === 'ECONNABORTED' && isFunction(options.timeoutCallback)) {
                        return options.timeoutCallback(error);
                    }
                    reject(error);
                });
        });

        return promise;
    };
});

