import { Action, Definition, State } from "./loadYml";

const EVENT_COLOR = "#LightYellow";
const HTTP_COLOR = "#Peru";
const SCRIPT_COLOR = "#Olive;text:white";
const BOT_COLOR = "#Blue;text:white";
const EMAIL_SENDER_COLOR = "#Red;text:white";
const DEFAULT_COLOR = '#Grey;text;white';

const EVENT_NAME = ": Event";
const HTTP_NAME = ": Http";
const SCRIPT_NAME = ": Script";
const BOT_NAME = ": Chat";
const EMAIL_SENDER_NAME = ": EmailSender";
const EXIT_NAME = ": Exit";

function isHttp(actions: Action[]): boolean {
  return actions
    .some((action) => action.actionType === "HttpRequest");
}

function isScript(actions: Action[]): boolean {
  return actions
    .some((action) => action.actionType === "Script");
}

function isChat(actions: Action[]): boolean {
  return actions
    .some((action) => action.actionType === 'Chat');
}

function isExit(actions: Action[]): boolean {
  return actions.some((action) => action.actionType === 'Exit');
}

function isEmailSender(actions: Action[]): boolean {
  return actions
    .some((action) => action.actionType === "EmailSender");
}

function isEmpty(arr: any[]): boolean {
  return arr === undefined || arr == null || arr.length === 0;
}

function isNotBlank(str: string): boolean {
  return str !== undefined && str !== null && str.trim() !== "";
}

function drawState(eventState: Map<string, Set<string>>, state: State): string {
  let sb = "";
  const name = state.name;
  const actions = state.actions;
  const handlers = state.eventHandlers;
  if (isEmpty(actions)) {
    if (isEmpty(handlers)) {
      throw Error(`state ${name} handlers must not empty`);
    }
    let events = new Set<string>();
    for (const handle of handlers) {
      const stateName = handle.eventType;
      events.add(stateName);
      sb += `state ${stateName} ${EVENT_COLOR} ${EVENT_NAME}\n`;
    }
    eventState.set(name, events);
  } else {
    if (isHttp(actions)) {
      sb += `state ${name} ${HTTP_COLOR} ${HTTP_NAME}\n`;
    } else if (isScript(actions)) {
      sb += `state ${name} ${SCRIPT_COLOR} ${SCRIPT_NAME}\n`;
    } else if (isChat(actions)) {
      sb += `state ${name} ${BOT_COLOR} ${BOT_NAME}\n`;
    } else if (isEmailSender(actions)) {
      sb += `state ${name} ${EMAIL_SENDER_COLOR} ${EMAIL_SENDER_NAME}\n`;
    } else {
      console.log('====', name, actions);
      // throw Error(`state ${name} actions is incorrect`);
    }
  }
  return sb;
}

function append(from: any, to: any, description: any) {
  if (isNotBlank(description)) {
    description = ": " + description;
  }
  return `${from} ---> ${to || ''}${description}\n`;
}

function wrapCond(cond: any) {
  if (isNotBlank(cond)) {
    return "(" + cond + ")";
  } else {
    return null;
  }
}

function drawTransitionCondition(eventCondition: any, transition: any) {
  const list = [];
  if (eventCondition !== undefined && eventCondition !== null) {
    list.push(eventCondition);
  }
  const transitionName = transition.transitionName;
  if (transitionName !== undefined && transitionName !== null) {
    list.push(`(TransitionName == ${transitionName})`);
  }
  const condition = transition.condition;
  if (condition !== undefined && condition !== null) {
    list.push(condition);
  }
  return list.join(" && ");
}

function drawExitCondition(eventCondition: any, exit: any) {
  const list = [];
  if (eventCondition !== undefined && eventCondition !== null) {
    list.push(eventCondition);
  }
  if (exit.condition !== undefined && exit.condition !== null) {
    list.push(exit.condition);
  }
  return list.join(" && ");
}

function drawStateLine(eventState: Map<string, Set<string>>, state: State) {
  const handlers = state.eventHandlers;
  if (isEmpty(handlers)) {
    return;
  }

  let sb = "";
  const name = state.name;
  for (const handler of handlers) {
    let sourceState;
    if (eventState.has(name)) {
      sourceState = handler.eventType;
    } else {
      sourceState = name;
    }
    const eventCondition = wrapCond(handler.condition);
    const actions = handler.actions;
    for (const action of actions) {
      const actionType = action.actionType;
      let targetState = action.targetState || '';
      if (actionType === "Transition") {
        if (eventState.has(targetState)) {
          let events = eventState.get(targetState);
          if (events && events.size > 1) {
            continue;
          } else if (events) {
            targetState = events.values().next().value;
          }
        }
        const desc = drawTransitionCondition(eventCondition, action);
        sb += append(sourceState, targetState, desc);
      } else if (actionType === "Exit") {
        const desc = drawExitCondition(eventCondition, action);
        sb += append(sourceState, "[*]", desc);
      }
    }
  }
  return sb;
}

export function transformDSL(definition: Definition): string {
  let sb = "";
  const eventState = new Map<string, Set<string>>();
  const states = definition["states"] || [];
  for (const state of states) {
    sb += drawState(eventState, state) || '';
  }
  for (const state of states) {
    sb += drawStateLine(eventState, state) || '';
  }
  return `@startuml

${sb}
@enduml
      `;
}

